@@ -15,25 +15,26 @@ class Conf(_config.ConfigNamespace):
     """
 
     server = _config.ConfigItem(
-        'http://irsa.ipac.caltech.edu/ibe/',
+        'https://irsa.ipac.caltech.edu/ibe/',
         'Name of the IBE server to use.')
     mission = _config.ConfigItem(
         'ptf',
         ('Default mission. See, for example, '
-         'http://irsa.ipac.caltech.edu/ibe/search/ for options.'))
+         'https://irsa.ipac.caltech.edu/ibe/search/ for options.'))
 
     dataset = _config.ConfigItem(
         'images',
         ('Default data set. See, for example, '
-         'http://irsa.ipac.caltech.edu/ibe/search/ptf for options.'))
+         'https://irsa.ipac.caltech.edu/ibe/search/ptf for options.'))
     table = _config.ConfigItem(
         'level1',
         ('Default table. See, for example, '
-         'http://irsa.ipac.caltech.edu/ibe/search/ptf/images for options.'))
+         'https://irsa.ipac.caltech.edu/ibe/search/ptf/images for options.'))
     timeout = _config.ConfigItem(
-        60,
+        120,
         'Time limit for connecting to the IRSA server.')
 
+
 conf = Conf()
 
 
