@@ -5,9 +5,9 @@ IBE
 
 API from
 
- http://irsa.ipac.caltech.edu/ibe/
+ https://irsa.ipac.caltech.edu/ibe/
 """
-from __future__ import print_function, division
+
 
 import os
 import webbrowser
@@ -15,7 +15,7 @@ from bs4 import BeautifulSoup
 
 import astropy.coordinates as coord
 from astropy.table import Table
-from astropy.extern import six
+import six
 
 from ..exceptions import InvalidQueryError
 from ..query import BaseQuery
@@ -276,7 +276,9 @@ class IbeClass(BaseQuery):
 
             root = BeautifulSoup(response.text)
             links = root.findAll('a')
-            missions = [os.path.basename(a.attrs['href']) for a in links]
+
+            missions = [os.path.basename(a.attrs['href'].rstrip('/'))
+                            for a in links]
             self._missions = missions
 
         return missions
