@@ -1,9 +1,9 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import print_function
+
 import os
 import requests
 
-from astropy.tests.helper import pytest
+import pytest
 from astropy.table import Table
 from astropy.coordinates import SkyCoord
 import astropy.units as u
@@ -12,16 +12,16 @@ from ...utils.testing_tools import MockResponse
 from ...ibe import Ibe
 
 DATA_FILES = {
-    ('http://irsa.ipac.caltech.edu/ibe/search/', None): 'missions.html',
-    ('http://irsa.ipac.caltech.edu/ibe/search/ptf/', None): 'datasets.html',
-    ('http://irsa.ipac.caltech.edu/ibe/search/ptf/images/',
+    ('https://irsa.ipac.caltech.edu/ibe/search/', None): 'missions.html',
+    ('https://irsa.ipac.caltech.edu/ibe/search/ptf/', None): 'datasets.html',
+    ('https://irsa.ipac.caltech.edu/ibe/search/ptf/images/',
      None): 'tables.html',
-    ('http://irsa.ipac.caltech.edu/ibe/search/ptf/images/level1',
+    ('https://irsa.ipac.caltech.edu/ibe/search/ptf/images/level1',
      frozenset((('FORMAT', 'METADATA'),))): 'columns.txt',
-    ('http://irsa.ipac.caltech.edu/ibe/search/ptf/images/level1',
+    ('https://irsa.ipac.caltech.edu/ibe/search/ptf/images/level1',
      frozenset((('where', 'expid <= 43010'), ('POS', '148.969687,69.679383'),
                 ('INTERSECT', 'OVERLAPS')))): 'pos.txt',
-    ('http://irsa.ipac.caltech.edu/ibe/search/ptf/images/level1',
+    ('https://irsa.ipac.caltech.edu/ibe/search/ptf/images/level1',
      frozenset((('where', "ptffield = 4808 and filter='R'"),
                 ('INTERSECT', 'OVERLAPS')))): 'field_id.txt'}
 
@@ -33,7 +33,10 @@ def data_path(filename):
 
 @pytest.fixture
 def patch_get(request):
-    mp = request.getfuncargvalue("monkeypatch")
+    try:
+        mp = request.getfixturevalue("monkeypatch")
+    except AttributeError:  # pytest < 3
+        mp = request.getfuncargvalue("monkeypatch")
     mp.setattr(requests.Session, 'request', get_mockreturn)
     return mp
 
