@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import print_function
+
 
 import re
 import os
@@ -11,9 +11,9 @@ import numpy as np
 import astropy.units as u
 import astropy.io.votable as votable
 from astropy import coordinates
-from astropy.extern import six
+import six
 from astropy.table import Table
-from astropy import log
+from astroquery import log
 from bs4 import BeautifulSoup
 
 from ..query import QueryWithLogin
@@ -46,10 +46,10 @@ def _validate_params(func):
                                  .format(Nrao.telescope_config))
         if isinstance(obs_band, (list, tuple)):
             for ob in obs_band:
-                if ob not in Nrao.obs_bands:
+                if ob.upper() not in Nrao.obs_bands:
                     raise ValueError("'obs_band' must be one of {!s}"
                                      .format(Nrao.obs_bands))
-        elif obs_band not in Nrao.obs_bands:
+        elif obs_band.upper() not in Nrao.obs_bands:
             raise ValueError("'obs_band' must be one of {!s}"
                              .format(Nrao.obs_bands))
         if sub_array not in Nrao.subarrays and sub_array != 'all':
@@ -80,6 +80,10 @@ class NraoClass(QueryWithLogin):
 
     obs_bands = ['ALL', 'all', '4', 'P', 'L', 'S', 'C', 'X', 'U', 'K', 'Ka', 'Q', 'W']
 
+    # we only ever use uppercase versions
+    telescope_config = [x.upper() for x in telescope_config]
+    obs_bands = [x.upper() for x in obs_bands]
+
     subarrays = ['ALL', 1, 2, 3, 4, 5]
 
     @_validate_params
@@ -392,11 +396,12 @@ class NraoClass(QueryWithLogin):
                                 freq_up=freq_up,
                                 telescope_config=telescope_config,
                                 obs_band=obs_band,
-                                sub_array=sub_array,
                                 querytype=querytype,
+                                sub_array=sub_array,
+                                project_code=project_code,
                                 protocol=protocol,
-                                get_query_payload=get_query_payload,
                                 retry=retry,
+                                get_query_payload=get_query_payload,
                                 cache=cache)
 
     def _parse_result(self, response, verbose=False):
