@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import print_function
+
 import os
 import requests
 
@@ -81,10 +81,11 @@ def test_query_region(patch_post, patch_parse_coordinates):
     assert isinstance(result, Table)
     assert len(result) > 0
     if 'Start Time' in result.colnames:
-        assert result['Start Time'][0] == b'83-Sep-27 09:19:30'
-    else:
-        assert result['Start_Time'][0] == b'83-Sep-27 09:19:30'
-    assert result['RA'][0] == b'04h33m11.096s'
+        truth = b'83-Sep-27 09:19:30' if commons.ASTROPY_LT_4_1 else '83-Sep-27 09:19:30'
+        assert result['Start Time'][0] == truth
+
+    truth = b'04h33m11.096s' if commons.ASTROPY_LT_4_1 else '04h33m11.096s'
+    assert result['RA'][0] == truth
 
 
 def test_query_region_archive(patch_post, patch_parse_coordinates):
@@ -105,3 +106,19 @@ def test_query_region_multiconfig(patch_post, patch_parse_coordinates):
         telescope_config=['A', 'AB', 'B', 'BC', 'C', 'CD', 'D'],
     )
     assert isinstance(result, Table)
+
+
+def test_query_region_lowercase(patch_post, patch_parse_coordinates):
+    # regression test for issue 1282
+    # test that the checker allows BnA, etc.
+    result = nrao.core.Nrao.query_region(
+        commons.ICRSCoordGenerator("05h35.8m 35d43m"), querytype='ARCHIVE',
+        telescope_config=['A', 'BnA', 'AB', 'B', 'BC', 'C', 'CD', 'D'],
+    )
+    assert isinstance(result, Table)
+
+    result = nrao.core.Nrao.query_region(
+        commons.ICRSCoordGenerator("05h35.8m 35d43m"), querytype='ARCHIVE',
+        obs_band=['K', 'Ka'],
+    )
+    assert isinstance(result, Table)
