@@ -10,10 +10,11 @@ import keyring
 import io
 import os
 import requests
+import textwrap
 
-from astropy.extern import six
+import six
 from astropy.config import paths
-from astropy.logger import log
+from astroquery import log
 import astropy.units as u
 from astropy.utils.console import ProgressBarOrSpinner
 import astropy.utils.data
@@ -34,14 +35,16 @@ def _replace_none_iterable(iterable):
     return tuple('' if i is None else i for i in iterable)
 
 
-class AstroQuery(object):
+class AstroQuery:
 
-    def __init__(self, method, url, params=None, data=None, headers=None,
-                 files=None, timeout=None):
+    def __init__(self, method, url,
+                 params=None, data=None, headers=None,
+                 files=None, timeout=None, json=None):
         self.method = method
         self.url = url
         self.params = params
         self.data = data
+        self.json = json
         self.headers = headers
         self.files = files
         self._hash = None
@@ -59,16 +62,20 @@ class AstroQuery(object):
             self._timeout = value
 
     def request(self, session, cache_location=None, stream=False,
-                auth=None, verify=True):
+                auth=None, verify=True, allow_redirects=True,
+                json=None):
         return session.request(self.method, self.url, params=self.params,
                                data=self.data, headers=self.headers,
                                files=self.files, timeout=self.timeout,
-                               stream=stream, auth=auth, verify=verify)
+                               stream=stream, auth=auth, verify=verify,
+                               allow_redirects=allow_redirects,
+                               json=json)
 
     def hash(self):
         if self._hash is None:
             request_key = (self.method, self.url)
-            for k in (self.params, self.data, self.headers, self.files):
+            for k in (self.params, self.data, self.json,
+                      self.headers, self.files):
                 if isinstance(k, dict):
                     entry = (tuple(sorted(k.items(),
                                           key=_replace_none_iterable)))
@@ -139,14 +146,15 @@ class LoginABCMeta(abc.ABCMeta):
 
 
 @six.add_metaclass(LoginABCMeta)
-class BaseQuery(object):
+class BaseQuery:
     """
     This is the base class for all the query classes in astroquery. It
     is implemented as an abstract class and must not be directly instantiated.
     """
 
     def __init__(self):
-        S = self._session = requests.session()
+        S = self._session = requests.Session()
+        self._session.hooks['response'].append(self._response_hook)
         S.headers['User-Agent'] = (
             'astroquery/{vers} {olduseragent}'
             .format(vers=version.version,
@@ -163,15 +171,54 @@ class BaseQuery(object):
         """ init a fresh copy of self """
         return self.__class__(*args, **kwargs)
 
-    def _request(self, method, url, params=None, data=None, headers=None,
+    def _response_hook(self, response, *args, **kwargs):
+        loglevel = log.getEffectiveLevel()
+
+        if loglevel >= 10:
+            # Log request at DEBUG severity
+            request_hdrs = '\n'.join(f'{k}: {v}' for k, v in response.request.headers.items())
+            request_log = textwrap.indent(
+                f"-----------------------------------------\n"
+                f"{response.request.method} {response.request.url}\n"
+                f"{request_hdrs}\n"
+                f"\n"
+                f"{response.request.body}\n"
+                f"-----------------------------------------", '\t')
+            log.debug(f"HTTP request\n{request_log}")
+        if loglevel >= 5:
+            # Log response at super-DEBUG severity
+            response_hdrs = '\n'.join(f'{k}: {v}' for k, v in response.headers.items())
+            if kwargs.get('stream'):
+                response_log = textwrap.indent(
+                    f"-----------------------------------------\n"
+                    f"{response.status_code} {response.reason} {response.url}\n"
+                    f"{response_hdrs}\n"
+                    "Streaming Data\n"
+                    f"-----------------------------------------", '\t')
+            else:
+                response_log = textwrap.indent(
+                    f"-----------------------------------------\n"
+                    f"{response.status_code} {response.reason} {response.url}\n"
+                    f"{response_hdrs}\n"
+                    f"\n"
+                    f"{response.text}\n"
+                    f"-----------------------------------------", '\t')
+            log.log(5, f"HTTP response\n{response_log}")
+
+    def _request(self, method, url,
+                 params=None, data=None, headers=None,
                  files=None, save=False, savedir='', timeout=None, cache=True,
-                 stream=False, auth=None, continuation=True, verify=True):
+                 stream=False, auth=None, continuation=True, verify=True,
+                 allow_redirects=True,
+                 json=None, return_response_on_save=False):
         """
         A generic HTTP request method, similar to `requests.Session.request`
         but with added caching-related tools
 
         This is a low-level method not generally intended for use by astroquery
-        end-users.
+        end-users.  However, it should _always_ be used by astroquery
+        developers; direct uses of `urllib` or `requests` are almost never
+        correct.
 
         Parameters
         ----------
@@ -180,6 +227,7 @@ class BaseQuery(object):
         url : str
         params : None or dict
         data : None or dict
+        json : None or dict
         headers : None or dict
         auth : None or dict
         files : None or dict
@@ -194,8 +242,16 @@ class BaseQuery(object):
         timeout : int
         cache : bool
         verify : bool
+            Verify the server's TLS certificate?
+            (see http://docs.python-requests.org/en/master/_modules/requests/sessions/?highlight=verify)
         continuation : bool
+            If the file is partly downloaded to the target location, this
+            parameter will try to continue the download where it left off.
+            See `_download_file`.
         stream : bool
+        return_response_on_save : bool
+            If ``save``, also return the server response. The default is to only
+            return the local file path.
 
         Returns
         -------
@@ -203,14 +259,19 @@ class BaseQuery(object):
             The response from the server if ``save`` is False
         local_filepath : list
             a list of strings containing the downloaded local paths if ``save``
-            is True
+            is True and ``return_response_on_save`` is False.
+        (local_filepath, response) : tuple(list, `requests.Response`)
+            a tuple containing a list of strings containing the downloaded local paths,
+            and the server response object, if ``save`` is True and ``return_response_on_save``
+            is True.
         """
         req_kwargs = dict(
             params=params,
             data=data,
             headers=headers,
             files=files,
-            timeout=timeout
+            timeout=timeout,
+            json=json
         )
         if save:
             local_filename = url.split('/')[-1]
@@ -218,18 +279,24 @@ class BaseQuery(object):
                 # Windows doesn't allow special characters in filenames like
                 # ":" so replace them with an underscore
                 local_filename = local_filename.replace(':', '_')
-            local_filepath = os.path.join(self.cache_location or savedir or '.', local_filename)
-            # REDUNDANT: spinner has this log.info("Downloading
-            # {0}...".format(local_filename))
-            self._download_file(url, local_filepath, cache=cache, continuation=continuation, method=method, auth=auth,
-                                **req_kwargs)
-            return local_filepath
+            local_filepath = os.path.join(savedir or self.cache_location or '.', local_filename)
+
+            response = self._download_file(url, local_filepath, cache=cache,
+                                           continuation=continuation, method=method,
+                                           allow_redirects=allow_redirects,
+                                           auth=auth, **req_kwargs)
+            if return_response_on_save:
+                return local_filepath, response
+            else:
+                return local_filepath
         else:
             query = AstroQuery(method, url, **req_kwargs)
             if ((self.cache_location is None) or (not self._cache_active) or (not cache)):
                 with suspend_cache(self):
                     response = query.request(self._session, stream=stream,
-                                             auth=auth, verify=verify)
+                                             auth=auth, verify=verify,
+                                             allow_redirects=allow_redirects,
+                                             json=json)
             else:
                 response = query.from_cache(self.cache_location)
                 if not response:
@@ -237,32 +304,55 @@ class BaseQuery(object):
                                              self.cache_location,
                                              stream=stream,
                                              auth=auth,
-                                             verify=verify)
+                                             allow_redirects=allow_redirects,
+                                             verify=verify,
+                                             json=json)
                     to_cache(response, query.request_file(self.cache_location))
             self._last_query = query
             return response
 
     def _download_file(self, url, local_filepath, timeout=None, auth=None,
-                       continuation=True, cache=False, method="GET", head_safe=False, **kwargs):
+                       continuation=True, cache=False, method="GET",
+                       head_safe=False, **kwargs):
         """
         Download a file.  Resembles `astropy.utils.data.download_file` but uses
         the local ``_session``
+
+        Parameters
+        ----------
+        url : string
+        local_filepath : string
+        timeout : int
+        auth : dict or None
+        continuation : bool
+            If the file has already been partially downloaded *and* the server
+            supports HTTP "range" requests, the download will be continued
+            where it left off.
+        cache : bool
+        method : "GET" or "POST"
+        head_safe : bool
         """
 
         if head_safe:
-            response = self._session.request("HEAD", url, timeout=timeout, stream=True,
+            response = self._session.request("HEAD", url,
+                                             timeout=timeout, stream=True,
                                              auth=auth, **kwargs)
         else:
-            response = self._session.request(method, url, timeout=timeout, stream=True,
+            response = self._session.request(method, url,
+                                             timeout=timeout, stream=True,
                                              auth=auth, **kwargs)
 
         response.raise_for_status()
         if 'content-length' in response.headers:
             length = int(response.headers['content-length'])
+            if length == 0:
+                log.warn('URL {0} has length=0'.format(url))
         else:
             length = None
 
-        if ((os.path.exists(local_filepath) and ('Accept-Ranges' in response.headers) and continuation)):
+        if ((os.path.exists(local_filepath)
+             and ('Accept-Ranges' in response.headers)
+             and continuation)):
             open_mode = 'ab'
 
             existing_file_length = os.stat(local_filepath).st_size
@@ -285,19 +375,18 @@ class BaseQuery(object):
                 self._session.headers['Range'] = "bytes={0}-{1}".format(existing_file_length,
                                                                         end)
 
-                response = self._session.request(method, url, timeout=timeout, stream=True,
+                response = self._session.request(method, url,
+                                                 timeout=timeout, stream=True,
                                                  auth=auth, **kwargs)
                 response.raise_for_status()
+                del self._session.headers['Range']
 
         elif cache and os.path.exists(local_filepath):
             if length is not None:
                 statinfo = os.stat(local_filepath)
                 if statinfo.st_size != length:
-                    log.warning("Found cached file {0} with size {1} that is "
-                                "different from expected size {2}"
-                                .format(local_filepath,
-                                        statinfo.st_size,
-                                        length))
+                    log.warning(f"Found cached file {local_filepath} with size {statinfo.st_size} "
+                                f"that is different from expected size {length}")
                     open_mode = 'wb'
                 else:
                     log.info("Found cached file {0} with expected size {1}."
@@ -311,12 +400,16 @@ class BaseQuery(object):
         else:
             open_mode = 'wb'
             if head_safe:
-                response = self._session.request(method, url, timeout=timeout, stream=True,
+                response = self._session.request(method, url,
+                                                 timeout=timeout, stream=True,
                                                  auth=auth, **kwargs)
                 response.raise_for_status()
 
         blocksize = astropy.utils.data.conf.download_block_size
 
+        log.debug(f"Downloading URL {url} to {local_filepath} with size {length} "
+                  f"by blocks of {blocksize}")
+
         bytes_read = 0
 
         # Only show progress bar if logging level is INFO or lower.
@@ -325,21 +418,18 @@ class BaseQuery(object):
         else:
             progress_stream = io.StringIO()
 
-        with ProgressBarOrSpinner(
-                length, ('Downloading URL {0} to {1} ...'
-                         .format(url, local_filepath)),
-                file=progress_stream) as pb:
+        with ProgressBarOrSpinner(length, f'Downloading URL {url} to {local_filepath} ...',
+                                  file=progress_stream) as pb:
             with open(local_filepath, open_mode) as f:
                 for block in response.iter_content(blocksize):
                     f.write(block)
-                    bytes_read += blocksize
+                    bytes_read += len(block)
                     if length is not None:
-                        pb.update(bytes_read if bytes_read <= length else
-                                  length)
+                        pb.update(bytes_read if bytes_read <= length else length)
                     else:
                         pb.update(bytes_read)
-
         response.close()
+        return response
 
 
 class suspend_cache:
