@@ -8,21 +8,20 @@ import tempfile
 import textwrap
 
 import astropy.coordinates as coord
-from astropy.extern.six.moves import urllib
-from astropy.extern import six
+from six.moves import urllib
+import six
 from astropy.io import fits
 import astropy.io.votable as votable
 import astropy.units as u
 from astropy.table import Table
-from astropy.tests.helper import remote_data
 import astropy.utils.data as aud
 
 from ...utils import chunk_read, chunk_report, class_or_instance, commons
 from ...utils.process_asyncs import async_to_sync_docstr, async_to_sync
-from ...utils.docstr_chompers import remove_returns, prepend_docstr_noreturns
+from ...utils.docstr_chompers import remove_sections, prepend_docstr_nosections
 
 
-class SimpleQueryClass(object):
+class SimpleQueryClass:
 
     @class_or_instance
     def query(self):
@@ -35,11 +34,12 @@ class SimpleQueryClass(object):
             return "instance"
 
 
-@remote_data
-def test_utils():
-    response = urllib.request.urlopen('http://www.ebay.com')
+@pytest.mark.remote_data
+def test_chunk_read():
+    datasize = 50000
+    response = urllib.request.urlopen(f'http://httpbin.org/stream-bytes/{datasize}')
     C = chunk_read(response, report_hook=chunk_report)
-    print(C)
+    assert len(C) == datasize
 
 
 def test_class_or_instance():
@@ -58,7 +58,7 @@ def test_parse_coordinates_1(coordinates):
     assert c is not None
 
 
-@remote_data
+@pytest.mark.remote_data
 @pytest.mark.parametrize(('coordinates'),
                          ["00h42m44.3s +41d16m9s",
                           "m81"])
@@ -72,9 +72,16 @@ def test_parse_coordinates_3():
         commons.parse_coordinates(9.8 * u.kg)
 
 
+def test_parse_coordinates_4():
+    # Regression test for #1251
+    coordinates = "251.51 32.36"
+    c = commons.parse_coordinates(coordinates)
+    assert c.to_string() == coordinates
+
+
 def test_send_request_post(monkeypatch):
     def mock_post(url, data, timeout, headers={}, status_code=200):
-        class SpecialMockResponse(object):
+        class SpecialMockResponse:
 
             def __init__(self, url, data, headers, status_code):
                 self.url = url
@@ -94,6 +101,7 @@ def test_send_request_post(monkeypatch):
     assert response.url == 'https://github.com/astropy/astroquery'
     assert response.data == dict(msg='ok')
     assert 'astroquery' in response.headers['User-Agent']
+    assert response.headers['User-Agent'].endswith("_testrun")
 
 
 def test_send_request_get(monkeypatch):
@@ -284,6 +292,10 @@ def test_async_to_sync(cls=Dummy):
 
 
 docstr3 = """
+    Parameters
+    ----------
+    first_param
+
     Returns
     -------
     Nothing!
@@ -301,10 +313,11 @@ docstr3_out = """
 
 
 def test_return_chomper(doc=docstr3, out=docstr3_out):
-    assert remove_returns(doc) == [x.lstrip() for x in out.split('\n')]
+    assert (remove_sections(doc, sections=['Returns', 'Parameters']) ==
+            [x.lstrip() for x in out.split('\n')])
 
 
-def dummyfunc():
+def dummyfunc1():
     """
     Returns
     -------
@@ -317,15 +330,28 @@ def dummyfunc():
     pass
 
 
+def dummyfunc2():
+    """
+    Returns
+    -------
+    Nothing!
+    """
+    pass
+
+
 docstr4 = """
     Blah Blah Blah
 
     Returns
     -------
     nothing
+
+    Examples
+    --------
+    no_examples_at_all
 """
 
-docstr4_out = """
+docstr4_out1 = """
     Blah Blah Blah
 
     Returns
@@ -337,14 +363,24 @@ docstr4_out = """
     Nada
 """
 
+docstr4_out2 = """
+    Blah Blah Blah
 
-def test_prepend_docstr(doc=docstr4, func=dummyfunc, out=docstr4_out):
-    fn = prepend_docstr_noreturns(doc)(func)
-    assert fn.__doc__ == textwrap.dedent(docstr4_out)
+    Returns
+    -------
+    Nothing!
+"""
+
+
+@pytest.mark.parametrize("func, out", [(dummyfunc1, docstr4_out1),
+                                       (dummyfunc2, docstr4_out2)])
+def test_prepend_docstr(func, out, doc=docstr4):
+    fn = prepend_docstr_nosections(doc, sections=['Returns', 'Examples'])(func)
+    assert fn.__doc__ == textwrap.dedent(out)
 
 
 @async_to_sync
-class DummyQuery(object):
+class DummyQuery:
 
     @class_or_instance
     def query_async(self, *args, **kwargs):
@@ -374,13 +410,26 @@ def patch_getreadablefileobj(request):
     # Monkeypatch hack: ALWAYS treat as a URL
     _is_url = aud._is_url
     aud._is_url = lambda x: True
+
+    if not commons.ASTROPY_LT_4_3:
+        _try_url_open = aud._try_url_open
+        aud._try_url_open = lambda x, **kwargs: MockRemote(x, **kwargs)
+
     _urlopen = urllib.request.urlopen
+    _urlopener = urllib.request.build_opener
+    _urlrequest = urllib.request.Request
     filesize = os.path.getsize(fitsfilepath)
 
-    class MockRemote(object):
+    class MockRemote:
         def __init__(self, fn, *args, **kwargs):
             self.file = open(fn, 'rb')
 
+        def __enter__(self):
+            return self
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            self.close()
+
         def info(self):
             return {'Content-Length': filesize}
 
@@ -391,13 +440,37 @@ def patch_getreadablefileobj(request):
             self.file.close()
 
     def monkey_urlopen(x, *args, **kwargs):
+        print("Monkeyed URLopen")
         return MockRemote(fitsfilepath, *args, **kwargs)
 
+    def monkey_builder(tlscontext=None):
+        mock_opener = type('MockOpener', (object,), {})()
+        mock_opener.open = lambda x, **kwargs: MockRemote(fitsfilepath, **kwargs)
+        return mock_opener
+
+    def monkey_urlrequest(x, *args, **kwargs):
+        # urlrequest allows passing headers; this will just return the URL
+        # because we're ignoring headers during mocked actions
+        print("Monkeyed URLrequest")
+        return x
+
+    aud.urllib.request.Request = monkey_urlrequest
+    aud.urllib.request.urlopen = monkey_urlopen
+    aud.urllib.request.build_opener = monkey_builder
     urllib.request.urlopen = monkey_urlopen
+    urllib.request.build_opener = monkey_builder
 
     def closing():
         aud._is_url = _is_url
+
+        if not commons.ASTROPY_LT_4_3:
+            aud._try_url_open = _try_url_open
+
         urllib.request.urlopen = _urlopen
+        aud.urllib.request.urlopen = _urlopen
+        urllib.request.build_opener = _urlopener
+        aud.urllib.request.build_opener = _urlopener
+        aud.urllib.request.Request = _urlrequest
 
     request.addfinalizer(closing)
 
@@ -405,7 +478,7 @@ def patch_getreadablefileobj(request):
 def test_filecontainer_save(patch_getreadablefileobj):
     ffile = commons.FileContainer(fitsfilepath, encoding='binary')
     temp_dir = tempfile.mkdtemp()
-    empty_temp_file = temp_dir + os.sep + 'test_emptyfile.fits'
+    empty_temp_file = f"{temp_dir}{os.sep}test_emptyfile.fits"
     ffile.save_fits(empty_temp_file)
     assert os.path.exists(empty_temp_file)
 
@@ -423,3 +496,11 @@ def test_filecontainer_get(patch_getreadablefileobj):
 def test_is_coordinate(coordinates, expected):
     out = commons._is_coordinate(coordinates)
     assert out == expected
+
+
+@pytest.mark.parametrize(('radius'),
+                         [0.01*u.deg, '0.01 deg', 0.01*u.arcmin]
+                         )
+def test_radius_to_unit(radius):
+    c = commons.radius_to_unit(radius)
+    assert c is not None
