@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import print_function
+
 
 import os
 import warnings
@@ -7,8 +7,8 @@ import json
 import copy
 import re
 
-from astropy.extern import six
-from astropy.extern.six import BytesIO
+import six
+from six import BytesIO
 import astropy.units as u
 import astropy.coordinates as coord
 import astropy.table as tbl
@@ -62,6 +62,16 @@ class VizierClass(BaseQuery):
             column descriptions defined on the Vizier web pages.
             See http://vizier.u-strasbg.fr/vizier/vizHelp/1.htx#ucd and
             http://cds.u-strasbg.fr/w/doc/UCD/
+        vizier_server : string
+            Name of the VizieR mirror to use.
+            (This parameter's default is set from a configuration object.)
+        timeout : number
+            timeout for connecting to server
+            (This parameter's default is set from a configuration object.)
+        row_limit : int
+            Maximum number of rows that will be fetched from the result
+            (set to -1 for unlimited).
+            (This parameter's default is set from a configuration object.)
         """
 
         super(VizierClass, self).__init__()
@@ -241,7 +251,7 @@ class VizierClass(BaseQuery):
 
         Parameters
         ----------
-        catalog : str or list, optional
+        catalog : str, Resource, or list, optional
             The catalog(s) that will be retrieved
 
         Returns
@@ -250,7 +260,7 @@ class VizierClass(BaseQuery):
             Returned if asynchronous method used
         """
 
-        if not isinstance(catalog, six.string_types):
+        if not isinstance(catalog, six.string_types + (votable.tree.Resource,)):
             catalog = list(catalog)
         data_payload = self._args_to_payload(catalog=catalog)
         if get_query_payload:
@@ -316,7 +326,7 @@ class VizierClass(BaseQuery):
     def query_region_async(self, coordinates, radius=None, inner_radius=None,
                            width=None, height=None, catalog=None,
                            get_query_payload=False, cache=True,
-                           return_type='votable'):
+                           return_type='votable', column_filters={}):
         """
         Serves the same purpose as `query_region` but only
         returns the HTTP response rather than the parsed result.
@@ -344,6 +354,9 @@ class VizierClass(BaseQuery):
         catalog : str or list, optional
             The catalog(s) which must be searched for this identifier.
             If not specified, all matching catalogs will be searched.
+        column_filters: dict, optional
+            Constraints on columns of the result. The dictionary contains
+            the column name as keys, and the constraints as values.
 
         Returns
         -------
@@ -354,19 +367,20 @@ class VizierClass(BaseQuery):
         catalog = VizierClass._schema_catalog.validate(catalog)
         center = {}
         columns = []
+
+        # Process coordinates
         if isinstance(coordinates, (commons.CoordClasses,) + six.string_types):
             c = commons.parse_coordinates(coordinates).transform_to('fk5')
 
             if not c.isscalar:
-                pos_list = []
+                center["-c"] = []
                 for pos in c:
                     ra_deg = pos.ra.to_string(unit="deg", decimal=True,
                                               precision=8)
                     dec_deg = pos.dec.to_string(unit="deg", decimal=True,
                                                 precision=8, alwayssign=True)
-                    pos_list += ["{}{}".format(ra_deg, dec_deg)]
-                center["-c"] = "<<;" + ";".join(pos_list)
-                columns += ["_q"]  # request a reference to the input table
+                    center["-c"] += ["{}{}".format(ra_deg, dec_deg)]
+                columns += ["_q"]  # Always request reference to input table
             else:
                 ra = c.ra.to_string(unit='deg', decimal=True, precision=8)
                 dec = c.dec.to_string(unit="deg", decimal=True, precision=8,
@@ -375,7 +389,7 @@ class VizierClass(BaseQuery):
         elif isinstance(coordinates, tbl.Table):
             if (("_RAJ2000" in coordinates.keys()) and ("_DEJ2000" in
                                                         coordinates.keys())):
-                pos_list = []
+                center["-c"] = []
                 sky_coord = coord.SkyCoord(coordinates["_RAJ2000"],
                                            coordinates["_DEJ2000"],
                                            unit=(coordinates["_RAJ2000"].unit,
@@ -385,15 +399,15 @@ class VizierClass(BaseQuery):
                                           precision=8)
                     dec_deg = dec.to_string(unit="deg", decimal=True,
                                             precision=8, alwayssign=True)
-                    pos_list += ["{}{}".format(ra_deg, dec_deg)]
-                center["-c"] = "<<;" + ";".join(pos_list)
-                columns += ["_q"]  # request a reference to the input table
+                    center["-c"] += ["{}{}".format(ra_deg, dec_deg)]
+                columns += ["_q"]  # Always request reference to input table
             else:
                 raise ValueError("Table must contain '_RAJ2000' and "
                                  "'_DEJ2000' columns!")
         else:
             raise TypeError("Coordinates must be one of: string, astropy "
                             "coordinates, or table containing coordinates!")
+
         # decide whether box or radius
         if radius is not None:
             # is radius a disk or an annulus?
@@ -423,16 +437,17 @@ class VizierClass(BaseQuery):
                 h_box = coord.Angle(height)
                 if w_box.unit != h_box.unit:
                     h_box = h_box.to(w_box.unit)
-                w_unit, w_value = _parse_angle(h_box)
-                h_unit, h_value = _parse_angle(w_box)
+                w_unit, w_value = _parse_angle(w_box)
+                h_unit, h_value = _parse_angle(h_box)
                 key = "-c.b" + w_unit
                 center[key] = "x".join([str(w_value), str(h_value)])
         else:
             raise Exception(
                 "At least one of radius, width/height must be specified")
 
+        # Prepare payload
         data_payload = self._args_to_payload(center=center, columns=columns,
-                                             catalog=catalog)
+                                             catalog=catalog, column_filters=column_filters)
 
         if get_query_payload:
             return data_payload
@@ -443,7 +458,7 @@ class VizierClass(BaseQuery):
         return response
 
     def query_constraints_async(self, catalog=None, return_type='votable',
-                                cache=True,
+                                cache=True, get_query_payload=False,
                                 **kwargs):
         """
         Send a query to Vizier in which you specify constraints with
@@ -502,6 +517,8 @@ class VizierClass(BaseQuery):
             catalog=catalog,
             column_filters=kwargs,
             center={'-c.rd': 180})
+        if get_query_payload:
+            return data_payload
         response = self._request(
             method='POST', url=self._server_to_url(return_type=return_type),
             data=data_payload, timeout=self.TIMEOUT, cache=cache)
@@ -515,35 +532,40 @@ class VizierClass(BaseQuery):
         body = OrderedDict()
         center = kwargs.get('center')
         # process: catalog
-        catalog = kwargs.get('catalog', self.catalog)
+        catalog = kwargs.get('catalog') or self.catalog
 
         if catalog is not None:
             if isinstance(catalog, six.string_types):
                 body['-source'] = catalog
             elif isinstance(catalog, list):
+                catalog = [item.name if hasattr(item, 'name') else item
+                           for item in catalog]
                 body['-source'] = ",".join(catalog)
+            elif hasattr(catalog, 'name'):
+                # this is probably a votable Resource, but no harm in duck-typing on `name`
+                body['-source'] = catalog.name
             else:
-                raise TypeError("Catalog must be specified as list or string")
+                raise TypeError("Catalog must be specified as list, string, or Resource")
         # process: columns
         columns = kwargs.get('columns', copy.copy(self.columns))
 
         if columns is not None:
             columns = self.columns + columns
+            # filter columns to _unique_ columns, preserving order in python >3.6
+            # note that "set" does not preserve order, but dict.keys does
+            columns = list(dict.fromkeys(columns, ).keys())
 
         # special keywords need to be treated separately
         # keyword names that can mean 'all'
         alls = ['all', '**']
         if any(x in columns for x in alls):
+            columns_all = True
             for x in alls:
                 if x in columns:
                     columns.remove(x)
-            body['-out.all'] = 2
-        # keyword name that means default columns
-        if '*' in columns:
-            columns.remove('*')
-            columns_default = True
+            body['-out.all'] = None
         else:
-            columns_default = False
+            columns_all = False
 
         # process: columns - identify sorting requests
         columns_out = []
@@ -558,16 +580,23 @@ class VizierClass(BaseQuery):
             else:
                 columns_out += [column]
 
-        if columns_default:
-            body['-out'] = '*'
-        else:
-            body['-out'] = columns_out
+        # calculated keyword names that start with an underscore
+        columns_calc = []
+        for column in columns_out:
+            if column[0] == '_':
+                columns_calc.append(column)
+        for column in columns_calc:
+            columns_out.remove(column)
 
-        if columns_out:
-            body['-out.add'] = ','.join(columns_out)
+        if columns_out and not columns_all:
+                body['-out'] = ','.join(columns_out)
+
+        if columns_calc:
+            body['-out.add'] = ','.join(columns_calc)
 
         if len(sorts_out) > 0:
             body['-sort'] = ','.join(sorts_out)
+
         # process: maximum rows returned
         row_limit = kwargs.get('row_limit') or self.ROW_LIMIT
         if row_limit < 0:
@@ -594,14 +623,26 @@ class VizierClass(BaseQuery):
         if ucd:
             body['-ucd'] = ucd
 
-        # create final script
-        script = "\n".join(["{key}={val}".format(key=key, val=val)
-                            for key, val in body.items()])
-        # add keywords
+        # create final script starting with keywords
+        script = []
         if (not isinstance(self.keywords, property) and
                 self.keywords is not None):
-            script += "\n" + str(self.keywords)
-        return script
+            script += [str(self.keywords)]
+        # add all items that are not lists
+        for key, val in body.items():
+            if type(val) is not list:
+                if val:
+                    script += ["{key}={val}".format(key=key, val=val)]
+                else:
+                    script += [key]
+        # add list at the end
+        for key, val in body.items():
+            if type(val) is list:
+                script += ["{key}=<<====AstroqueryList".format(key=key)]
+                script += val
+                script += ["====AstroqueryList"]
+        # merge result
+        return "\n".join(script)
 
     def _parse_result(self, response, get_catalog_names=False, verbose=False,
                       invalid='warn'):
