@@ -5,8 +5,7 @@ from numpy import testing as npt
 import pytest
 from astropy.table import Table
 import astropy.units as u
-from astropy.extern import six
-from astropy.extern.six.moves import urllib_parse as urlparse
+import six
 from ... import vizier
 from ...utils import commons
 from ...utils.testing_tools import MockResponse
@@ -17,7 +16,8 @@ if six.PY3:
 VO_DATA = {'HIP,NOMAD,UCAC': "viz.xml",
            'NOMAD,UCAC': "viz.xml",
            'B/iram/pdbi': "afgl2591_iram.xml",
-           'J/ApJ/706/83': "kang2010.xml"}
+           'J/ApJ/706/83': "kang2010.xml",
+           "find_2009ApJ...706...83K": "find_kangapj70683.xml"}
 
 
 def data_path(filename):
@@ -39,9 +39,24 @@ def post_mockreturn(self, method, url, data=None, timeout=10, files=None,
                     params=None, headers=None, **kwargs):
     if method != 'POST':
         raise ValueError("A 'post request' was made with method != POST")
-    datad = dict([urlparse.parse_qsl(d)[0] for d in data.split('\n')])
-    filename = data_path(VO_DATA[datad['-source']])
-    content = open(filename, "rb").read()
+    if isinstance(data, dict):
+        datad = data
+    else:
+        datad = {}
+        for line in data.split('\n'):
+            if '=' in line:
+                key, value = line.split('=', maxsplit=1)
+                datad[key.strip()] = value.strip()
+            else:
+                datad[line] = None
+    if '-source' in datad:
+        # a request for the actual data
+        filename = data_path(VO_DATA[datad['-source']])
+        content = open(filename, "rb").read()
+    elif '-words' in datad:
+        # a find_catalog request/only metadata
+        filename = data_path(VO_DATA['find_' + datad['-words']])
+        content = open(filename, "rb").read()
     return MockResponse(content, **kwargs)
 
 
@@ -168,6 +183,31 @@ def test_get_catalogs(patch_post):
     assert isinstance(result, commons.TableList)
 
 
+def test_find_resource_then_get(patch_post):
+    reses = vizier.core.Vizier.find_catalogs('2009ApJ...706...83K')
+    res = next(iter(reses.values()))
+
+    result = vizier.core.Vizier.get_catalogs(res)
+    assert isinstance(result, commons.TableList)
+    assert next(iter(result.keys())).startswith(res.name)
+
+    resultl = vizier.core.Vizier.get_catalogs([res])
+    assert type(result) == type(resultl)
+    assert len(result) == len(resultl)
+    assert len(result[0]) == len(resultl[0])
+
+
+def test_catalog_consistency_issue1326(patch_post):
+    # regression test for issue 1326
+    result1 = vizier.core.Vizier(catalog='J/ApJ/706/83').query_constraints_async(testconstraint='blah',
+                                                                                 get_query_payload=True)
+    result2 = vizier.core.Vizier().query_constraints_async(testconstraint='blah',
+                                                           catalog='J/ApJ/706/83',
+                                                           get_query_payload=True)
+
+    assert result1 == result2
+
+
 class TestVizierKeywordClass:
 
     def test_init(self):
